/// @file ilmsens_hal_types.h 
/// @brief Type definitions for Ilmsens HAL
///
/// This header defines structure types used by the various setup and 
/// information retrieval HAL functions. Furthermore, it defines the
/// data types for measured samples and register/memory access.
///
/// @author Ralf Herrmann <ralf.herrmann@ilmsens.com>

/* ILMsens UWB HAL API

   This header contains common type definitions used in the API calls.
   
   Platform: C/C++
   Author  : R. Herrmann
   
   Revision:
     
     1.0.0: (01/2017)

     + initial revision containing module info type.
*/

#pragma once

//get std. types
#include <stdint.h>

///////////////////////////////////////////////////////////////////////////////////////////////////
// global HAL type definitions
///////////////////////////////////////////////////////////////////////////////////////////////////

/// \ingroup ilmsens_hal_version
/// Represents the version identifier.
///
/// Hint: In case useless data is returned or memory access exceptions occur,
///       make sure your compiler does not use wrong memory alignement for structure members.
struct ilmsens_hal_Version
{
  unsigned int mMajor;  ///< major version number
  unsigned int mMinor;  ///< minor version number
  unsigned int mBuild;  ///< build number
};

/// \ingroup ilmsens_hal_config
/// Represents the basic device configuration
///
/// Hint: In case useless data is returned or memory access exceptions occur,
///       make sure your compiler does not use wrong memory alignement for structure members.
struct ilmsens_hal_ModConfig
{
  unsigned int mOrder;        ///< MLBS order
  unsigned int mSub;          ///< subsampling factor
  double mClk;                ///< master clock
  unsigned int mOV;           ///< number of oversampling
  unsigned int mTx;           ///< number of transmitters
  unsigned int mRx;           ///< number of receivers
};

/// \ingroup ilmsens_hal_modinfo
/// Represents the device configuration & runtime parameters and limits
///
/// Hint: In case useless data is returned or memory access exceptions occur,
///       make sure your compiler does not use wrong memory alignement for structure members.
struct ilmsens_hal_ModInfo 
{
  ilmsens_hal_ModConfig mConfig;    ///< basic configuration of sensor
  double mTB_Fc;              ///< corner frequency for timebase calibration
  double mTemp;               ///< device temperature
  double mLSB_Volt;           ///< ADC LSB voltage (for calculating physical values out of raw data)
  double mFSR[2];             ///< ADC full scale range (minimum and maximum value)
  unsigned int mHWAvg;        ///< number of hardware averages (FPGA)
  unsigned int mAvg;          ///< number of averages
  unsigned int mAvgLim[2];    ///< limits for averages (minimum and maximum value)
  unsigned int mWait;         ///< number of wait cycles
  unsigned int mWaitLim[2];   ///< limits for wait cycles (minimum and maximum value)
  unsigned int mNumSamp;      ///< number of samples per channel
};

/// \ingroup ilmsens_hal_meas_data
/// Represents the data type of raw measured data
typedef int32_t ilmsens_hal_SampleType;

/// \ingroup ilmsens_hal_lowlevel
/// Represents data type for register and memory access
typedef uint32_t ilmsens_hal_MemoryType;
